/*
             LUFA Library
     Copyright (C) Dean Camera, 2017.
  dean [at] fourwalledcubicle [dot] com
           www.lufa-lib.org
*/
/*
  Copyright 2017  Dean Camera (dean [at] fourwalledcubicle [dot] com)
  Permission to use, copy, modify, distribute, and sell this
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.
  The author disclaims all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/
#ifndef __DATAFLASH_USER_H__
#define __DATAFLASH_USER_H__
    /* Includes: */
        // TODO: Add any required includes here
    /* Preprocessor Checks: */
        #if !defined(__INCLUDE_FROM_DATAFLASH_H)
            #error Do not include this file directly. Include LUFA/Drivers/Board/Dataflash.h instead.
        #endif
    /* Private Interface - For use in library only: */
    #if !defined(__DOXYGEN__)
        /* Macros: */
            #define DATAFLASH_CHIPCS_MASK                // TODO: Replace this with a mask of all the /CS pins of all Dataflashes
            #define DATAFLASH_CHIPCS_DDR                 // TODO: Replace with the DDR register name for the board's Dataflash ICs
            #define DATAFLASH_CHIPCS_PORT                // TODO: Replace with the PORT register name for the board's Dataflash ICs
    #endif
    /* Public Interface - May be used in end-application: */
        /* Macros: */
            #define DATAFLASH_TOTALCHIPS                 1 // TODO: Replace with the number of Dataflashes on the board, max 2
            #define DATAFLASH_NO_CHIP                    0
            #define DATAFLASH_CHIP1                      // TODO: Replace with mask with the pin attached to the first Dataflash /CS set
            #define DATAFLASH_CHIP2                      // TODO: Replace with mask with the pin attached to the second Dataflash /CS set
            #define DATAFLASH_PAGE_SIZE                  // TODO: Replace with the page size for the Dataflash ICs
            #define DATAFLASH_PAGES                      // TODO: Replace with the total number of pages inside one of the Dataflash ICs
        /* Inline Functions: */
        #if !defined(__DOXYGEN__)
            static inline void Dataflash_Init(void)
            {
                DATAFLASH_CHIPCS_DDR  |= DATAFLASH_CHIPCS_MASK;
                DATAFLASH_CHIPCS_PORT |= DATAFLASH_CHIPCS_MASK;
            }
            static inline uint8_t Dataflash_TransferByte(const uint8_t Byte) ATTR_ALWAYS_INLINE;
            static inline uint8_t Dataflash_TransferByte(const uint8_t Byte)
            {
                // TODO
            }
            static inline void Dataflash_SendByte(const uint8_t Byte) ATTR_ALWAYS_INLINE;
            static inline void Dataflash_SendByte(const uint8_t Byte)
            {
                // TODO
            }
            static inline uint8_t Dataflash_ReceiveByte(void) ATTR_ALWAYS_INLINE ATTR_WARN_UNUSED_RESULT;
            static inline uint8_t Dataflash_ReceiveByte(void)
            {
                // TODO
            }
            static inline uint8_t Dataflash_GetSelectedChip(void) ATTR_ALWAYS_INLINE ATTR_WARN_UNUSED_RESULT;
            static inline uint8_t Dataflash_GetSelectedChip(void)
            {
                return (~DATAFLASH_CHIPCS_PORT & DATAFLASH_CHIPCS_MASK);
            }
            static inline void Dataflash_SelectChip(const uint8_t ChipMask) ATTR_ALWAYS_INLINE;
            static inline void Dataflash_SelectChip(const uint8_t ChipMask)
            {
                DATAFLASH_CHIPCS_PORT = ((DATAFLASH_CHIPCS_PORT | DATAFLASH_CHIPCS_MASK) & ~ChipMask);
            }
            static inline void Dataflash_DeselectChip(void) ATTR_ALWAYS_INLINE;
            static inline void Dataflash_DeselectChip(void)
            {
                Dataflash_SelectChip(DATAFLASH_NO_CHIP);
            }
            static inline void Dataflash_SelectChipFromPage(const uint16_t PageAddress)
            {
                Dataflash_DeselectChip();
                if (PageAddress >= (DATAFLASH_PAGES * DATAFLASH_TOTALCHIPS))
                  return;
                #if (DATAFLASH_TOTALCHIPS == 2)
                    if (PageAddress & 0x01)
                      Dataflash_SelectChip(DATAFLASH_CHIP2);
                    else
                      Dataflash_SelectChip(DATAFLASH_CHIP1);
                #else
                    Dataflash_SelectChip(DATAFLASH_CHIP1);
                #endif
            }
            static inline void Dataflash_ToggleSelectedChipCS(void)
            {
                uint8_t SelectedChipMask = Dataflash_GetSelectedChip();
                Dataflash_DeselectChip();
                Dataflash_SelectChip(SelectedChipMask);
            }
            static inline void Dataflash_WaitWhileBusy(void)
            {
                Dataflash_ToggleSelectedChipCS();
                Dataflash_SendByte(DF_CMD_GETSTATUS);
                while (!(Dataflash_ReceiveByte() & DF_STATUS_READY));
                Dataflash_ToggleSelectedChipCS();
            }
            static inline void Dataflash_SendAddressBytes(uint16_t PageAddress,
                                                          const uint16_t BufferByte)
            {
                #if (DATAFLASH_TOTALCHIPS == 2)
                    PageAddress >>= 1;
                #endif
                Dataflash_SendByte(PageAddress >> 5);
                Dataflash_SendByte((PageAddress << 3) | (BufferByte >> 8));
                Dataflash_SendByte(BufferByte);
            }
        #endif
#endif
